/*#######################################################################################
Dump-Unterprogramme zur Ausgabe eines Speicherdumps
Test der im EEPROM abgelegten IP-Adresse 

Copyright (C) 2005 Joachim Brke

#######################################################################################*/

#include "../main.h"
#include "dump.h"
#include "../OWI/OWIPolled.h"
#include "../OWI/OWIcrc.h"

extern void checkEEPROMip (void);

//*************************************************************************************************
// IP-Adresse ist jetzt einstellbar	jb 26.07.2005
// Einstellungen werden im EEPROM gesichert jb 22.10.2005 
//
void checkEEPROMip(void)
{
	int i;			// Laufvariable
	unsigned char Feld[8];	// temporres Feld, um CRC-Unterprogram nutzen zu knnen
	
	for (i=0; i<8; i++)		// alle acht Bytes (IP mit CRC-Summe) ins RAM kopieren
		Feld[i]=eeprom_read_byte((char *)IPoffset+i);		// IP, Variablen und CRC aus dem EEPROM lesen

	if (!OWI_ComputeCRCn(Feld,8))	{				// Wenn CRC ok,
		for (i=0; i<4; i++)	MYIP[i]=Feld[i];	//  die 4 Bytes von MYIP berschreiben
		VA3				= Feld[4];					//  Anzeigeoptionen setzen
		DALLAS[0][0]	= Feld[5];					//  aktive Busleitungen setzen
		VAR				= Feld[6];					//  refresh-Zeit fr Temperaturmessungen setzen
	}
	else {											// wenn CRC nicht ok, default-Werte benutzen
		VA3				= 0;						//  alle Bausteine anzeigen
		DALLAS[0][0]	= OWI_PIN_5;				//  aktive Busleitung ist D5 (evtl. mehrere durch | verknpfen)
		VAR				= 3;						//  refresh-Zeit fr Temperaturmessungen auf 3 s setzen
	}
	
	DALLAS[0][0] &= erlaubteLeitungen;		// es sind nur bestimmte Busleitungen zulssig!
	BUSES = DALLAS[0][0];
	
	VAF	= 0;	// Funlayer auf 0 setzen;
	VAP	= 2;	// Bildgre auf 320*240 setzen (Schalter=2)
	VAB = 1;	// EEPROM-Dump
	
//	VAX = 0;	// Ausgangsbits auf "0" setzen
	VAY = 0;
	VAZ = 0;
	VAhA =0;
	VAhB =0;
	VAhC =0;
	VAhD =0;
	VAhE =0;
/*	// Das Testmuster dient zum Prfen der Speicher-Dump-Routinen
	if (eeprom_read_byte((char *)768)!=0)		// Wenn das Testmuster nicht vorhanden ist
		for (i=0; i<256; i++)							// fr 256 Bytes
			eeprom_write_byte((char *)i+768,i&0xff);	// Testmuster schreiben */
}

//*************************************************************************************************
// Ausgeben eines Speicherdumps auf der seriellen Schnittstelle
//
//	bytes enthlt die Anzahl der Bytes, die ausgegeben werden sollen
//	*Datenlesen zeigt auf die Funktion, die die nchsten 16 Byte zur Verfgung stellt. Es gibt zwei
//	Funktionen, eine liest die Daten aus dem EEPROM des AVR und die Andere aus dem EEPROM der 
//	Netzwerkkarte.
//
void dump (int bytes, void (*Datenlesen)(int, char *))
{
	int j=0;									// Adresszhler
	unsigned char zeile[80];					// Puffer fr die Ausgabezeile

	Adresszeile (zeile,0);						// Adrezeile anlegen
	printf_P(PSTR("%s"),zeile);				//  und ausgeben

	for (j=0; j<bytes; j+=16) {				// insgesamt Bytes/16 Datenzeilen ausgeben
		Datenzeile (zeile,0,j,*Datenlesen);	// Datenzeile erstellen
		printf_P(PSTR("%s"),zeile);			//  und ausgeben
	}
	printf_P(PSTR("\n"));						// zum Abschlu eine Leerzeile ausgeben
}

//*************************************************************************************************
// Initialisieren des Ausgabepuffers fr die Zeile eines Speicherdumps
//
//	*Zeile zeigt auf den Buffer, zindex gibt die Anfangsposition im Buffer an
//  Der Buffer wird mit Leerzeichen gefllt und mit Zeilen- und Stringende versehen. Zwischen den 
//	beiden 8-Byte-Blcken wird ein '-' ausgegeben. Die Daten werden spter an der richtigen Stelle
//	in den Puffer eingesetzt.
//
void Initzeile (char *zeile, int zindex)
{
	unsigned char i;
	for(i=0;i<79;i++) zeile[zindex+i]=' ';	// Zeile mit Leerzeichen fllen
	zeile[zindex+30] = '-';					// in der Mitte "-" als Trenner vorsehen
	zeile[zindex+77] = 0x0d;					// CR setzen
	zeile[zindex+78] = 0x0a;					// LF setzen
	zeile[zindex+79] = 0;						// Stringende setzen
}

//*************************************************************************************************
// Ausgeben eines Bytes in Form von zwei Hexadezimalzeichen in einen Buffer
//	*buffer zeigt auf den Buffer
//  xx enthlt das Datenbyte, das ausgegeben werden soll
//	aa enthlt die Position im Buffer
//
void hex_zeile (char *buffer,char xx, int aa)
{
	unsigned char x;
	x = xx >> 4;
	buffer[aa++] = x>9 ? x+'A'-10 : x+'0';
	x = xx & 0x0f;
	buffer[aa] = x>9 ? x+'A'-10 : x+'0'; 
}

//*************************************************************************************************
// Bereitstellen einer Datenzeile fr einen Dump
//
//	*Zeile zeigt auf den Buffer
//	zindex gibt die Anfangsposition im Buffer an
//	adr gibt die aktuelle Adresse des Dump-Bereiches an
//	*Datenlesen zeigt auf die Funktion, die die nchsten 16 Datenbyte liefert
//
void Datenzeile (char *zeile, int zindex, int adr, void (*Datenlesen)(int, char *))
{
	unsigned char k,l,Zwischenspeicher[16];
	
	Initzeile (zeile,zindex);				// Ausgabepuffer initialisieren
	(*Datenlesen) (adr, Zwischenspeicher);	// 16 Byte einlesen
	hex_zeile(zeile,adr/256, zindex);		// HOB der Adresse an den Zeilenanfang setzen
	hex_zeile(zeile,adr&0xff,zindex+2);	// LOB der Adresse schreiben
	zeile[zindex+4] = ':';					// Adresse mit Doppelpunkt abschlieen

	for(k=0;k<=7;k++)  hex_zeile(zeile,Zwischenspeicher[k],zindex+k*3+6);	// HEX-Zeichen 0..7 drucken
	for(k=8;k<=15;k++) hex_zeile(zeile,Zwischenspeicher[k],zindex+k*3+8);	// HEX-Zeichen 8..f drucken

	for(k=0; k<=15; k++) {				// am Zeilenende Daten als 16 ASCII-Zeichen ausgeben
		l = Zwischenspeicher[k];			// einzelnes Zeichen untersuchen und HTML-konform machen
		if ((l<32)|(l>126)) l='.';		// nicht druckbare Zeichen als "." ausgeben
		if (l=='<') l='.';				// '<' wird wegen des Browsers als '.' dargestellt!
		zeile[zindex+k+57]=l;				// Zeichen im Buffer platzieren
	} 
}

//*************************************************************************************************
// Bereitstellen der Adresszeile, die als "berschrift" ausgegeben wird
//
//	*Zeile zeigt auf den Anfang des Buffers
//	zindex zeigt auf die Position, ab der geschrieben werden soll
//
void Adresszeile (char *zeile, int zindex)
{
	unsigned char k;
	Initzeile (zeile,zindex);			// Ausgabepuffer initialisieren
	for(k=0;k<=7;k++)  hex_zeile(zeile,k,zindex+k*3+6);	// Adressen 0..7
	for(k=8;k<=15;k++) hex_zeile(zeile,k,zindex+k*3+8);	// Adressen 8..f
	zeile[zindex+67] = 0x0d;			// CR setzen (eine Leerzeile nach der Adresszeile einfgen)
	zeile[zindex+68] = 0x0a;			// LF setzen (deshalb ein weiterer Zeilenwechsel)
}

//*************************************************************************************************
// Lesen von 16 Byte fr den 93LC46-Speicherdump
//	Der Parameter dummy enthlt die Adresse, die hier wegen der sequentiellen Speicherung nicht
//	bentigt wird. *Z zeigt auf das Datenfeld, in dem die Daten zurckgegeben werden.
//
void read9346 (int dummy, char *Z)
{
	unsigned char k;
	for (k=0; k<=15; k++) Z[k]=0;			// Datenfeld initialisieren 
		
	for (k=0; k<=127; k++) {					// 128 bit aus dem EEPROM der Netzwerkkarte in das Datenfeld lesen
		WriteRTL (CR9346,(EEM1 |EECS ));		// Takt = 0	
		WriteRTL (CR9346,(EEM1 |EECS |EESK));	// Takt = 1
		Z[(k>>3)^1] |= (ReadRTL(CR9346)&EEDO)<<(~k&7);// Bit in das Datenfeld einfgen
	}
	// Mit der MAC-Adresse der Netzwerkkarte die default-Einstellung des Feldes "mymac[]" berschreiben
	if (dummy==0)	for (k=0; k<=5; k++)	mymac[k]=Z[k+4];
}

//*************************************************************************************************
// Lesen von 16 Byte fr den EEPROM-Speicherdump
//	Adresse enthlt die Adresse des ersten zu lesenden Bytes. *Z zeigt auf das Datenfeld, 
//	in dem die Daten zurckgegeben werden.
//
void readEEPROM (int Adresse, char *Z)
{
	unsigned char k;
	for (k=0; k<=15; k++) Z[k]=eeprom_read_byte((char *)Adresse+k);			
}
