/*#######################################################################################
AVR Small Webserver 

Copyright (C) 2004 Ulrich Radig

Bei Fragen und Verbesserungen wendet euch per EMail an

mail@ulrichradig.de

oder im Forum meiner Web Page : www.ulrichradig.de


Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
GNU General Public License, wie von der Free Software Foundation verffentlicht, 
weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
(nach Ihrer Option) jeder spteren Version. 

Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
Programm erhalten haben. 
Falls nicht, schreiben Sie an die Free Software Foundation, 
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#######################################################################################*/

#include "main.h"


// Die MAC Addresse des Web Servers wird spter aus dem EEPROM der Netzwerkkarte geladen
// Die Nummern 1..6 dienen nur als Platzhalter
//
unsigned char mymac[] = {1,2,3,4,5,6};

// Die Standard IP Addresse des Web Servers kann mit der Adresse aus dem EEPROM
// berschrieben werden. Die Adresse 192.168.99.199 mu beim ersten Zugriff eingestellt
// sein. Die Adresse ist ber das HTML-Interface einstellbar.
//
unsigned char MYIP[] = {192,168,0,108};





//############################################################################
//Hauptprogramm
int main (void)
//############################################################################
{
#if USE_SERVO
	servo_init();
#else
	// Initialisierung der LED-(PWM)-Ansteuerung an Pin 7 Port D
	LED_LINES		&= ~(1<<LED_PIN);	// Datenleitung auf "0" setzen
	LED_LINES_DIR	|= (1<<LED_PIN);	// Richtungsregister: LED_PIN = Output
	TCCR2	= 0x63;	// Phase correct PWM Mode fr OC2, CLK/32 d.h. 980 Hz Signalfrequenz bei 16 MHz Takt
	VA2		= 26;	// Startwert = 10% Helligkeit
	OCR2	= VA2;	// PWM-Register auf den Startwert setzen
#endif
	
	UART_Init();	// Initialisierung der seriellen Schnittstelle

#if USE_MMC	
	while (mmc_init() != 0) 		//ist der Rckgabewert ungleich NULL ist ein Fehler aufgetreten
		printf ("** Keine MMC/SD-Karte gefunden! **\r\n");// die MMC/SD-Karte wird dann neu Initialisiert
	printf ("MMC/SD-Karte gefunden!\r\n");
	Cluster_Data_Store();	//Speichern von Cluster Size und Cluster Offset (Anfang des Datenbereichs)
#endif

	// Initialisierung der Portleitungen
	PORTB	&= ~(1<<4);			// Datenleitung auf "0" setzen
	DDRB	|= (1<<4);				// Port B 4 = Ausgang;
	PORTA	&= ~(1<<5|1<<6);		// Datenleitung auf "0" setzen
	DDRA	|= (1<<5|1<<6); 		// Port A 5,6 = Ausgang;

	Network_init();			//Initialisierung der Netzwerkkarte
	printf_P(PSTR("RTL8019AS: "));
	if(ReadRTL(0) == 34)	printf_P (PSTR("Okay 34")); 
		else 				printf_P (PSTR("ERROR(%d)"),ReadRTL(0));

	// Ausgabe der MAC Adresse auf der seriellen Schnittstelle
	printf_P (PSTR("\r\nMAC: %02X:%02X:%02X:%02X:%02X:%02X\r\n"),
		mymac[0],mymac[1],mymac[2],mymac[3],mymac[4],mymac[5]);

	checkEEPROMip();	// Prfung, ob im EEPROM Einstelldaten vorhanden sind

	// Ausgabe der IP Adresse auf der seriellen Schnittstelle
	printf_P (PSTR("IP : %d.%d.%d.%d\r\n"),MYIP[0],MYIP[1],MYIP[2],MYIP[3]);	
	// Variablen zum spteren Einstellen der IP-Adresse initialisieren
	VAi=MYIP[0];	VAj=MYIP[1];	VAk=MYIP[2];	VAl=MYIP[3];
	
	// Speicherdump des AVR-EEPROMs auf der seriellen Schnittstelle ausgeben
	printf_P (PSTR("\nEEPROM Dump:\r\n"));	// berschrift ausgeben
	dump (128,*readEEPROM);				// EEPROM-Speicherdump mit 128 Byte ausgeben (1024 mglich) 
	
	//Interrupt 4 bei Mega103/128 oder Interrupt 0 bei Mega32, soll bei ansteigender Flanke auslsen
	Networkcard_INT_Set();		// Jede nderung lst einen Interrupt aus
	Networkcard_INT_Enable();	// Interrupt INT0 fr die Netzwerkkarte einschalten

	Clock_init();		// Uhr initialisieren und starten

	OWI_init();			// one-wire Schnittstelle initialisieren

#if USE_MCA25_CAM
	mca25_init();		// Kamera initialisieren
	mca25_configure();	// und konfigurieren
#endif

	sei();				// Globale Interrupts Einschalten
	
	while(1)			// Hauptprogramm luft stndig in einer Schleife
	{
		OWImain();		// one-wire Schnittstelle bedienen
			
		
	}

	return (1);
}
